import {
  Component,
  Input,
  ViewChild,
  ViewContainerRef,
} from "@angular/core";
import {
  SohoAutoCompleteComponent,
  SohoFileUploadAdvancedComponent,
  SohoModalDialogRef,
  SohoModalDialogService,
  SohoToastService,
} from "@infor/sohoxi-angular";
import {
  DialogService,
  IIonApiRequestOptions,
  IWidgetContext,
  IWidgetInstance,
  WidgetMessageType,
  WidgetState,
  ILanguage,
} from "lime";
import { interval, Observable } from "rxjs";
import { map, startWith } from "rxjs/operators";
export class headingCsv {
  actualCol: string;
  newCol: string;
}
@Component({
  template: `
    <div #dialogPlaceholder >
      <div class="field">
        <input
          soho-autocomplete
          [source]="source"
          filterMode="contains"
          [(ngModel)]="selected"
          (selected)="onSelected($event)"
          (ngModelChange)="modelChanged($event)"
          [placeholder] = "placeHolderText"
          [attr.data-lmw-id]="autoId"
          [attr.attributes]="[{ name: 'data-lmw-id', value: autoId }]"
        />
      </div>
      
      <div >
       <input  soho-checkbox type="checkbox"  [attr.id]="conversionChkBx" name="checkbox1" [(ngModel)]="convertTOJSONORNDJSON" (change)="onCheckboxChange($event)">
       <label soho-label [attr.for]="conversionChkBx" [forCheckBox]="true" class="inline-block-child" >{{ language?.get("desConvert")}}</label>
       <div class="inline-block-child" [style.display]="hideConversion">
        <label for="lblConversionTechnique"></label>
        <select soho-dropdown soho-busyindicator noSearch name="lblConversionTechnique" [(disabled)] = "selectOptionDisabled"
          (change)="onChange($event)"   >
          <option *ngFor="let option of getConversionTechniques()" [value]="option">{{option}} 
          </option>
        </select>
      </div>
     
      <div class="field">
        <label class="labelRed">{{ language?.get("desMaxfiles")}}</label>
        <label class="labelRed">{{ language?.get("desConvertNdJson")}}</label>
        </div>
        <soho-fileupload-advanced
         data-options="{allowedTypes: 'xls|xlsx|csv|xml|json'}" 
          [send]="onSend" [(maxFileSize)] = "maxFileSize" [(errorMaxFileSize)] = "errorMessageFileSize"
          (fileremoved)="onFileRemoved($event)"
        ></soho-fileupload-advanced>
      </div>

      <div class="modal-buttonset">
        <button class="btn-modal" (click)="modalRef.close()">Cancel</button>
        <button
          class="btn-modal-primary"
          (click)="submitBtnClick(); modalRef.close()"
          [disabled]="fileUploadDisabled"
        >
          Submit
        </button>
      </div>
    </div>
  `,
    styles: ['.labelRed { color: red; }','.inline-block-child { display: inline-block; margin-right: 10px ;margin-bottom: 5px ;}','.div_Height { height: 100px;}']
})

export class FAIModalComponent {
  @ViewChild(SohoFileUploadAdvancedComponent, { static: true })
  @ViewChild("dialogPlaceholder", { read: ViewContainerRef, static: true })
  @ViewChild(SohoAutoCompleteComponent)
   autoId: string;
  fileUploadAdvanced?: SohoFileUploadAdvancedComponent;
  hideConversion  = "none"
  public xssSource: SohoAutoCompleteSource = [];
  autocomplete?: SohoAutoCompleteComponent;
  public selectOptionDisabled = false;
  public selected: any;
  public fileUploadDisabled = false;
  options?: SohoAutoCompleteOptions;
  widgetContext: IWidgetContext;
  widgetInstance: IWidgetInstance;
  modalRef: SohoModalDialogRef<FAIModalComponent>;
  uploadFile: File;
  fileArrays: File[]= [];
  errorFile = "";
  fileStatus:SohoFileUploadAdvancedStatus;
  jsonORNDJsonValue :string;
  conversionChkBx : string;
  convertTOJSONORNDJSON : false;
  maxFileSize = 0;
  errorMessageFileSize = "";
  isNDJSON = true;
  fileType = "";
  readerResult : any;
  language: ILanguage;
  sohodialogService: SohoModalDialogService;
  dialogservice: DialogService;
  placeHolderText : string;


  constructor(
    dialogservice: DialogService,
    sohodialogService: SohoModalDialogService,
    private readonly viewRef: ViewContainerRef,
    private toastService: SohoToastService
  )
   {
    this.sohodialogService = sohodialogService;
    this.dialogservice = dialogservice;
    this.toastService = toastService;
  }

  ngOnInit() {
    this.setEnable();
    this.conversionChkBx = "chkBox" + this.widgetContext.getWidgetInstanceId();
    this.language = this.widgetContext.getLanguage();
    var fileSize = this.widgetContext.getSettings().getString("fileSize")
    this.placeHolderText = this.language.get("desTypeToSearch")
    if(fileSize == "5 MB"){
      this.errorMessageFileSize = this.language.get("desErrorFiveMB");
      this.maxFileSize = 5000000;
    }
    else {
      this.errorMessageFileSize = this.language.get("desErrorTenMB");
      this.maxFileSize = 10000000;
    }
  }
  public source = (term: string, response: any) => {
    response(term, this.xssSource);
  };
  public onSelected(event: any) {
    this.setEnable();
  }
  public modelChanged(event : any) {
    if(this.selected == ""){
    this.setEnable();
    }
  }
  onSend = (_formData: FormData, status: SohoFileUploadAdvancedStatus) => {
    this.uploadFile = status.file;
    this.fileArrays.push(status.file);
    this.fileStatus = status;
    if(this.isValidCSVFile(this.uploadFile.name) || this.isValidXMLFile(this.uploadFile.name)
    || this.isValidXLSXFile(this.uploadFile.name) || this.isValidJSONFile(this.uploadFile.name) ){
      this.setEnable();
      status.setCompleted();

    }
    else {
      status.setAbort();
        this.widgetContext.showWidgetMessage({
          message: this.language.get("desUploadErrorFile"),
          type: WidgetMessageType.Alert,
        });
    }
    if (this.uploadFile.name == null) {
      status.setAbort();
    }
  };
  isValidCSVFile(file: string) {
    return file.toLowerCase().endsWith(".csv");
  }
  isValidXLSXFile(file: string) {
    return file.toLowerCase().endsWith(".xls") || file.toLowerCase().endsWith(".xlsx");
  }
  isValidJSONFile(file: string) {
    return file.toLowerCase().endsWith(".json") ;
  }
  isValidXMLFile(file: string) {
    return file.toLowerCase().endsWith(".xml");
  }
  onCheckboxChange(event : any){
    if(event.target.checked){
      this.hideConversion = "block";
    }
    else {
      this.hideConversion = "none";
    }
  }
  secondsSinceLastSelect?: Observable<number>;
  resetSelectCount() {
    this.secondsSinceLastSelect = interval(1000).pipe(
      map((n) => n + 1),
      startWith(0)
    );
  }
  submitBtnClick() {
    if (this.isValidCSVFile(this.uploadFile.name) ||this.isValidXLSXFile(this.uploadFile.name) || this.isValidJSONFile(this.uploadFile.name) || this.isValidXMLFile(this.uploadFile.name)) {
      this.setBusy(true);
      if(this.convertTOJSONORNDJSON)
      {
        const reader = new FileReader();
        reader.readAsDataURL(this.uploadFile);
        reader.onload = () => {
          this.readerResult =  reader.result;
          
          if (this.isValidCSVFile(this.uploadFile.name)) {
            this.fileType = "CSV";
          }
          else if (this.isValidXLSXFile(this.uploadFile.name)) {
            this.fileType = "EXCEL";
          }
          else if (this.isValidXMLFile(this.uploadFile.name)) {
            this.fileType = "XML";
          }
          else if (this.isValidJSONFile(this.uploadFile.name)) {
            this.fileType = "JSON";
          }
          else {
            
            this.widgetContext.showWidgetMessage({
              message: this.language.get("desUploadErrorFile"),
              type: WidgetMessageType.Alert,
            });
          }
          this.convertToScript(reader.result);
      };
        
      }
      else {
        this.uploadFileToMultiPart();
      }
    }  else {
      this.widgetContext.showWidgetMessage({
        message:
           this.language.get("desFileType") + " : " +
          this.uploadFile.name.substr(
            this.uploadFile.name.lastIndexOf(".") + 1,
            this.uploadFile.name.length
          ) +
          " : "  +  this.language.get("desPublishFailed"),
        type: WidgetMessageType.Error,
      });
    }
  }
  convertToScript(base64:string | ArrayBuffer | null) {
    this.setBusy(true);
    var newarr = base64.toString().split(",");
    let parameterRequest ='{ \"inputfile\": \"' +newarr[1] +'",\"\inputFileType":\"'+this.fileType+'\" ,\"isNDJSON\":'+this.isNDJSON+'}';
    let request = this.createScriptRequest(parameterRequest);
    this.widgetContext.executeIonApiAsync<any>(request).subscribe(
      (response) => {
        let res = response.data;
        let outputResponse = JSON.parse(res);
        this.jsonORNDJsonValue = outputResponse.outputfile;
        if (response.status == 201 || response.status == 200) {
          this.errorFile = "";
          this.setBusy(false);
          this.uploadFileToMultiPart();
        }
      },
      (error) => {
        this.errorFile = this.language.get("desFailedParseData");
        this.fileStatus.setCompleted();
        this.setBusy(false);
        var baseMessage = this.language.get("desFailedData");
       var message = this.getErrorMessage(error,baseMessage);
          this.widgetContext.showWidgetMessage({
            message: message,
            type: WidgetMessageType.Error,
          });
      }
    );
  }
  getErrorMessage(error : any,baseMessage : string) : string{
    var message  = baseMessage;
    if (error.status == 400) {
      this.setBusy(false);
      let errorMessage   = JSON.parse(error.error);
      let errorMessageValue = errorMessage.message;
      if(errorMessageValue.toLowerCase().includes("5 mb")){
          message = this.language.get("desUploadLesserSize");
      }
      else {
          message = errorMessageValue;
      }
      return message;
    }
      if (error.status == 401) { 
       message = message + " : " + this.language.get("desUnAuthorized");
     }
     else if(error.status == 404) {
       message = message + " : " + this.language.get("desNotFound");
     }
     else if(error.status == 405) {
       message = message + " : " + this.language.get("desMethodNotAllowed");
     }
     else if(error.status == 412) {
       message = message + " : " + this.language.get("desPreconditionFailed");
     }
     else if(error.status == 415) {
       message = message + " : " + this.language.get("desUnsupportedMediaType");
     }
     else if(error.status == 500) {
       message = message + " : "  + this.language.get("desInternalServerError");
     }
     else {
       message = this.language.get("desUnableToGetResponse");
     }
 
     return message;
   }

  createDataLakeRequestMultiPartUpload(
    formData?: FormData,
    headers?: object
  ): IIonApiRequestOptions {
    if (!headers) {
      headers = { Accept: "*/*" };
    }
    let jsonData: any;
    const url = "CustomerApi/IMSWrapper/send/v2/multipartMessage";
    this.setBusy(true);
    const request: IIonApiRequestOptions = {
      method: "POST",
      url: url,
      cache: false,
      headers: headers,
      data: formData,
      responseType: "text" as const,
    };

    return request;
  }
 uploadFileToMultiPart() {
    let parameterRequest =
      '{"documentName" : "' +
      this.selected +
      '","messageId" : "'+this.uploadFile.name+'","fromLogicalId" : "' +
      "lid://" +
      this.widgetContext.getSettings().getString("imslogicalid") +
      '","toLogicalId" : "lid://default","encoding" : "NONE","characterSet" : "UTF-8","documentId" : "' +
      this.uploadFile.name +
      '"}';
    const fd = new FormData();
    fd.append("ParameterRequest", parameterRequest);
    if(this.convertTOJSONORNDJSON){
      fd.append("MessagePayload", this.jsonORNDJsonValue);
    }
    else {
    fd.append("MessagePayload", this.uploadFile);
    }
    let request = this.createDataLakeRequestMultiPartUpload(fd);
    this.widgetContext.executeIonApiAsync<any>(request).subscribe(
      (response) => {
        let res = response.data;
        if (response.status == 201 || response.status == 200) {
          this.setBusy(false);
          this.widgetContext.showWidgetMessage({
            message: this.language.get("desPublishedSuccess"),
            type: WidgetMessageType.Info,
          });
        }
      },
      (error) => {
        this.setBusy(false);
          this.widgetContext.showWidgetMessage({
            message: this.getErrorMessage(error,this.language.get("desPublishFail")),
            type: WidgetMessageType.Error,
          });
       
      }
    );
  }
  onFileRemoved(file: File) {
    this.RemoveElementFromStringArray(file)
    if(this.fileArrays.length > 0)
    {
    this.uploadFile = this.fileArrays[this.fileArrays.length - 1];
    }
    else {
      this.uploadFile = undefined;
    }

    this.toastService.show({
      title: this.language.get("desFileRemoved"),
      message: file && file.name ? file.name : "",
    });
   
      this.setEnable();
    
  }
  setEnable() {
    if(this.uploadFile &&  this.isValidJSONFile(this.uploadFile.name)) {
      this.selectOptionDisabled = true;
    }
    else {
    this.selectOptionDisabled = false;
    }

    if(this.fileArrays.length == 1 && this.selected && typeof this.selected!='undefined' && this.selected != "")
    {
    (this.submitBtnClick as any).disabled = false;
    this.fileUploadDisabled = (this.submitBtnClick as any).disabled;
    }
    else {
      (this.submitBtnClick as any).disabled = true;
      this.fileUploadDisabled = (this.submitBtnClick as any).disabled;
    }
  }
  private setBusy(isBusy: boolean): void {
    this.widgetContext.setState(
      isBusy ? WidgetState.busy : WidgetState.running
    );
  }
  createScriptRequest(
    xmlString?: string,
    headers?: object
  ): IIonApiRequestOptions {
    if (!headers) {
      headers = { Accept: "*/*", "Content-type": "application/json" };
    }
    let jsonData: any;
    const url = "/IONSERVICES/scriptingservice/engine/v1/scripts/GenericFileUploadJSONConversion/execute";
    this.setBusy(true);
    const request: IIonApiRequestOptions = {
      method: "POST",
      url: url,
      cache: false,
      headers: headers,
      data: xmlString,
      responseType: "text" as const,
    };
    return request;
  }
  onChange(event: SohoDropDownEvent) {
    if(event.data == "NDJSON"){
      this.isNDJSON = true;
    }
    else {
      if(this.uploadFile && this.isValidJSONFile(this.uploadFile.name)){
        this.isNDJSON = true;
      }
      else {
      this.isNDJSON = false;
      }
    }
    
  }
  getConversionTechniques():string[] {
       return [
      this.language.get("desNDJSON"),
      this.language.get("desJSON"),
    ];
  }
   RemoveElementFromStringArray(element: any) {
    this.fileArrays.forEach((value,index)=>{
        if(value.name == element.name) this.fileArrays.splice(index,1);
    });
}
}

